using LeedsBeerQuest.API.Data.Models;
using LeedsBeerQuest.API.Data.Services.Interfaces;
using Microsoft.AspNetCore.Mvc;

namespace LeedsBeerQuest.API.Controllers;

[ApiController]
[Route("[controller]")]
public class VenueController : ControllerBase
{
    private readonly IVenueService _venueService;

    public VenueController(IVenueService venueService)
    {
        _venueService = venueService;
    }

    [ProducesResponseType(typeof(IEnumerable<Venue>), StatusCodes.Status200OK)]
    [HttpGet]
    public async Task<IActionResult> GetVenues(
        [FromQuery] string? search = null,
        [FromQuery] string[]? categories = null,
        [FromQuery] string[]? tags = null)
    {
        var venues = await _venueService.GetFilteredVenues(search, tags, categories);

        return Ok(venues);
    }

    [ProducesResponseType(typeof(Venue), StatusCodes.Status200OK)]
    [ProducesResponseType(StatusCodes.Status400BadRequest)]
    [ProducesResponseType(StatusCodes.Status404NotFound)]
    [HttpGet("{venueId:int}")]
    public async Task<IActionResult> GetVenue(int venueId)
    {
        if (venueId < 1)
        {
            return BadRequest("Requested venue ID must be more than zero");
        }

        var venue = await _venueService.GetVenue(venueId);

        return venue == null ? NotFound() : Ok(venue);
    }
}