import { InputLabel, ListItemText, MenuItem, Select, SelectChangeEvent } from "@mui/material";
import { useState } from "react";

export type TFilter = {
    id: number;
    name: string;
}

export type TDropdownFilterProps = {
    prefillText: string;
    filters: TFilter[];
    setSelectedFilters: (filters: TFilter[]) => void;
}

export function DropdownFilter(props: TDropdownFilterProps) {
    const [selectedFilterStrings, setSelectedFilterStrings] = useState<string[]>([]);

    // Handle changes to selections for the dropdown.
    const handleChange = (event: SelectChangeEvent<string[]>) => {
        const { value } = event.target;

        const filterStrings = typeof value === "string" ? value.split(",") : value;
        setSelectedFilterStrings(filterStrings);

        updateSelectedFilters(filterStrings);
    };

    const updateSelectedFilters = (stringFilters: string[]): void => {
        const currentFilters: TFilter[] = [];

        // Dropdown returns strings, we need to find the tags related to their names here.
        stringFilters.forEach((filterString) => {
            filterString = filterString.trim();
            const foundFilter = props.filters.find((filter) => filter.name === filterString);

            if (foundFilter) {
                currentFilters.push(foundFilter);
            }
        });

        props.setSelectedFilters(currentFilters);
    }

    return(
        <div>
            <InputLabel>{props.prefillText}</InputLabel>
            <Select
            style={{ minWidth: 150, maxWidth: 300 }}
            multiple
            value={selectedFilterStrings}
            onChange={handleChange}
            renderValue={(selected) => selected.join(', ')}>
                {props.filters.map((filter) => (
                    <MenuItem key={filter.id} value={filter.name}>
                        <ListItemText primary={filter.name} />
                    </MenuItem>
                ))}
        </Select>
        </div>
    );
}