import { ArrayToArrayUrlParams } from "./helpers";
import { TCategory, TTag, TVenue, TVenueFilters } from "./types";

export class API {
    constructor(private baseAPIUrl: string) {}

    getVenues = async (venueFilters: TVenueFilters | null): Promise<TVenue[]> => {
        // Build up the params for the request here.
        let params = "";
        params += ArrayToArrayUrlParams("categories", venueFilters?.categories.map((c) => c.name) ?? []);
        params += ArrayToArrayUrlParams("tags", venueFilters?.tags.map((v) => v.name) ?? [], params.length < 1);

        // NameFilter params here, logic included to init with ? or &.
        params += venueFilters?.nameFilter ? `${params.length < 1 ? "?" : "&"}search=${venueFilters.nameFilter}` : "";

        const response = await fetch(`${this.baseAPIUrl}/venue${params}`);
        return await response.json();
    }

    getCategories = async (): Promise<TCategory[]> => {
        const response = await fetch(`${this.baseAPIUrl}/category`);
        return await response.json();
    }

    getTags = async (): Promise<TTag[]> => {
        const response = await fetch(`${this.baseAPIUrl}/tag`);
        return await response.json();
    }
}